const PeriodsDay = window.nguiAsyncUrlImport('/service/intelli/v1/filedb/scripts/maxpermissiblevalues/components/periodsDay.js');

window.ngui(() => {
    return {
        emits: [
            'form:submit',
        ],
        props: [
            'job', 
        ],
        data() {            
            return {
                intervalOptions: [
                    { label: 'hourly', value: 3600 },
                    { label: 'daily', value: -1 },
                    // {label: 'weekly', value: -2},
                ],
                jobExecutionTargetOptions: [
                    { label: 'local', value: 'local' },
                    { label: 'cluster', value: 'cluster' },
                ],                
                jobLabel: '',
                jobExecutionTarget: 'local',
                interpolationUpdateKey: 0,
                // explanatoryDatapointsUpdateKey starts at 1 to fix warning "Duplicate keys found during update: 1 Make sure keys are unique."
                explanatoryDatapointsUpdateKey: 1, 
                selectionMethod: '',
                dateSelectorConfig: [
                    {
                        selectionMethod: 'movingWindows',
                        items: [
                        {
                            id: 'trainingDates',
                            opacity: 0.3,
                            color: '#abdbe3',
                            dates: [],
                            maxOneElement: true,
                        },
                        ],
                    },
                    {
                        selectionMethod: 'isolatedWindows',
                        items: [
                        {
                            id: 'trainingDates',
                            opacity: 0.3,
                            color: '#abdbe3',
                            dates: [],
                            maxOneElement: false,
                        },
                        {
                            id: 'measureDates',
                            opacity: 0.3,
                            color: '#fefef',
                            dates: [],
                            maxOneElement: false,
                        },
                        ],
                    },
                ],
                chartNodes: [],
                loading: {
                    main: false,
                    featureImportance: false,
                },
                dialog: {
                    chart: false,
                    featureImportance: false,
                },
                importantFeatureDependentVarUuid: '',
                timeVars: [
                    'hod',
                    'dow',
                    'woy',
                    'month',
                    'quarter',
                    'Mo2Sa-So',
                    'Mo2Fr-Sa-So',
                    'Mo2Fr-Sa2So',
                ],        
                // jobType: 'maxpermissiblevalues',
                jobConfig: {
                    interval: undefined,
                    trainTime: '00:00',
                    nodes: [],
                    importantFeaturesSets: [],
                    trainMethod: 'movingWindows',
                    trainingDates: [],
                    measureDates: [],
                    periodsDay: {
                        train: 60,
                        retrain: 20,
                    },
                },                
                nodeCollectionDefault: {
                    uuid: '', // Wil be unique ????,
                    dependentVariable: false,
                    enabled: true,
                    chartOptions: {
                        show: true,
                        zIndex: 100,
                    },
                    interpolation: {
                        template: 'rawValues',
                        method: {
                            diff: '',
                            name: '',
                            type: '',
                            fill: '',
                            statistic: '',
                        },
                        filterParams: [],
                        normalizeParams: [],
                        prospective: false,
                      },
                      pmModel: {
                        path: '',
                        processManagerUuid: '',
                        processUuid: '',
                        instanceUuid: '',
                    },
                    extInfos: {
                        name: '',
                        accType: '',
                        unit: '',
                    },
                    bins: {
                        base: 0,
                        step: 3,
                    },
                    mandatory: false,                               
                }
            }
        },
        methods: {
            getActiveNodes() {
                return this.jobConfig.nodes.filter((node) => {
                    return node.enabled;
                });
            },
            computeFeatureImportance() {
                // Strip node content to the minimal since they are passed over cli args
                return new Promise((resolve, reject) => {
                    const enabledNodes = this.getActiveNodes();
                    const nodes = enabledNodes.map((node) =>  {
                        return {
                            uuid: node.uuid,
                            dependentVariable: node.dependentVariable,
                            interpolation: node.interpolation,
                            bins: node.bins,
                            pmModel: node.pmModel,
                            extInfos: node.extInfos
                        }
                    });

                    // to do -> should we normalize dates in intelli?
                    const normalizedDates = this.jobConfig.trainingDates.map((date) => this.ctx.intelli.getLocalizedDateTimeByInterval(date, this.jobConfig.interval));
                    const intelliExecutePayload = {
                        scriptFile: 'metrics/compute_feature_importance.py',
                        scriptType: 'maxpermissiblevalues',
                        inputData: {
                            dmsData: {
                                nodes,
                                dates: normalizedDates,
                                interval: this.jobConfig.interval
                            }
                        },
                    };

                    this.ctx.intelli.execute(intelliExecutePayload)
                    .then((response) => {
                        const intelliResponse = response.data;
                        if ('error' in intelliResponse && intelliResponse.error !== null) {
                            reject(data.error);
                        }
                        const importantFeatures = JSON.parse(intelliResponse.data);

                        this.jobConfig.importantFeaturesSets = [];
                        // Add active flags and set to false after each computation
                        importantFeatures.forEach((set) => {
                            set.independentVars = set.independentVars.map((item) => {
                                return { ...item, active: false};
                            });
                            this.jobConfig.importantFeaturesSets.push(set);
                        });
                        resolve(true);
                    })
                    .finally(() => {
                        this.importantFeatureDependentVarUuid = this.jobConfig.importantFeaturesSets[0].dependentVar.uuid;
                    })
                    .catch((err) => {
                        reject(err);
                  });
                });
            },
            enableFeatureImporanceTest() {
                const dependentNodes = this.jobConfig.nodes.filter((node) => {
                    return (node.enabled && node.dependentVariable);               
                });
                return (
                    dependentNodes.length > 0  && 
                    (this.jobConfig.interval !== undefined && this.jobConfig.interval !== '') &&
                    this.jobConfig.trainingDates.length > 0                    
                );
            },
            getTrainMethodOptions() {
                return this.dateSelectorConfig.map((option) => ({ label: option.selectionMethod, value: option.selectionMethod }));
            },          
            validateAndSave(startJob = false, closeDialog = true) {
                return new Promise((resolve, reject) => {
                    if (this.$refs.jobForm) {
                        const validator = this.$refs.jobForm.formValidator();
                        validator()
                        .then((success) => {
                            if (success) {   
                                this.job.config = this.jobConfig;
                                this.job.label = this.jobLabel;
                                this.job.executionTarget = this.jobExecutionTarget;

                                // Create inputTimeSeries (Data passed over disk)
                                const nodes = this.jobConfig.nodes.map((node) => {
                                    return {
                                        dependentVariable: node.dependentVariable,
                                        pmModel: node.pmModel,                      
                                        interpolation: node.interpolation,
                                        bins: node.bins,
                                    }    
                                });

                                this.job.inputTimeSeries = {
                                    nodes,
                                    dates: this.jobConfig.trainingDates.concat(this.jobConfig.measureDates),
                                    interval: this.jobConfig.interval,
                                }

                                this.$emit('form:submit', { job: this.job, startJob, closeDialog });
                                resolve(success);
                            }
                        })
                        .catch((err) => {
                            console.error(err);
                            reject(err);
                        });                
                    }
                });
            },
            loadChart() {
                this.loading.main = true;
                this.chartNodes = [];
                return Promise.all(this.jobConfig.nodes.map(async (explanatoryNode) => {
                    const ep = await this.ctx.processManager.fetchEndpoint(explanatoryNode.pmModel);
                    if (explanatoryNode.enabled) {
                        this.chartNodes.push(
                        {
                            node: { url: ep, pathes: [explanatoryNode.pmModel.path] },
                            interpolation: explanatoryNode.interpolation,
                            extInfos: explanatoryNode.extInfos,
                        });
                    }
                }))
                .catch((err) => {
                    log.debug(err);
                })
                .finally(() => {
                    this.dialog.chart = true;
                    this.loading.main = false;
                });                
            },
        },
        beforeMount() {
            this.jobLabel = this.job.label;
            this.jobExecutionTarget = this.job.executionTarget;
            // populate on edit
            if (JSON.stringify(this.job.config) !== '{}') {
                this.jobConfig = this.job.config;                
            }
            this.selectionMethod = this.jobConfig.trainMethod;
            this.dateSelectorConfig[0].items[0].dates = this.jobConfig.trainingDates;
            this.dateSelectorConfig[1].items[0].dates = this.jobConfig.trainingDates;
            this.dateSelectorConfig[1].items[1].dates = this.jobConfig.measureDates;

            if (this.jobConfig.importantFeaturesSets.length > 0) {
                this.importantFeatureDependentVarUuid = this.getActiveNodes().filter((node) => {return (node.dependentVariable)})[0].uuid;
            }
        },
        mounted() {
            // Update node names from DMS
            if (this.jobConfig.nodes.length > 0) {
                this.jobConfig.nodes.map((node) => {                
                    node.extInfos = this.ctx.intelli.fetchExtInfos(node.pmModel)
                    .then((extInfos) => {
                        node.extInfos = extInfos;
                    }).catch((err) => {
                        console.error(err);
                    });
                });
            }
        },
        watch: {
            dateSelectorConfig: {
                deep: true,
                handler(newValue) {
                    const option = newValue.filter((opt) => opt.selectionMethod === this.selectionMethod)[0];
                    option.items.forEach((item) => {
                      if (item.id === 'trainingDates') {
                        this.jobConfig.trainingDates = item.dates;
                      }
                      if (item.id === 'measureDates') {
                        this.jobConfig.measureDates = item.dates;
                      }
                    });
                }
            },
        },
        nodes() {
            return [         
                { 
                    type: 'NgDialog',
                    props: {
                        title: 'dateSelector',
                        modelValue: this.dialog.chart,
                        medium: false,
                        noOk: true,
                        noCancel: true,
                        draggable: true,
                        maximizing: true,
                        width: '1000px',
                        'onUpdate:modelValue': (value) => {
                            this.dialog.chart = value;
                        },
                    },
                    children: {
                        default: (slot) => {
                            return [
                                {
                                    type: 'NgBIDateSelector',
                                    props: {
                                        nodes: this.chartNodes,
                                        dateSelectorOptions: this.dateSelectorConfig,
                                        selectionMethod: this.selectionMethod,
                                        'onUpdate:selectionMethod': (value) =>{
                                            this.selectionMethod = value;
                                            this.jobConfig.trainMethod = value;
                                        },
                                        enableTime: false,
                                        interval: this.jobConfig.interval,
                                        'onUpdate:interval': (value) => {
                                            this.jobConfig.interval= value;
                                        },
                                        intervalOptions: this.intervalOptions,                              
                                        'onUpdate:chartNodes': (nodes) => {
                                            nodes.forEach((node, i) => {
                                                this.jobConfig.nodes[i].interpolation = node.interpolation;
                                            });
                                            // Set two keys to avoid warning
                                            // Duplicate keys found during update: 1 Make sure keys are unique.
                                            this.interpolationUpdateKey++;
                                            this.explanatoryDatapointsUpdateKey++;
                                        },                                 
                                    },    
                                },                                 
                            ];                         
                        }
                    }
                },
                { 
                    type: 'NgDialog',
                    props: {
                        title: 'mostImportantFeatures',
                        modelValue: this.dialog.featureImportance,
                        medium: false,
                        noOk: false,
                        noCancel: true,
                        draggable: true,
                        maximizing: true,
                        width: '800px',
                        'onUpdate:modelValue': (value) => {
                            this.dialog.featureImportance = value;
                        },
                    },
                    children: {
                        default: (slot) => {
                            const vNodes = [                               
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12 '                                                        
                                    },
                                    children: {
                                        default:() => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                    },
                                                    children: {
                                                        default: () => {
                                                            return [                                                             
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        label: 'dependentVariable',
                                                                        type: 'select',
                                                                        modelValue: this.importantFeatureDependentVarUuid,
                                                                        options: this.getActiveNodes().filter((node) => {
                                                                            return (node.dependentVariable);
                                                                        }).map((node) => {
                                                                            return {
                                                                                label: `${this.ctx.intelli.getNodeName(node.extInfos)} - ${node.pmModel.path}`,
                                                                                value: node.uuid,
                                                                            }                                                                            
                                                                        }),
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.importantFeatureDependentVarUuid = value;                                                                                             
                                                                        },                                 
                                                                    },                                        
                                                                },
                                                                { 
                                                                    type: 'NgActionButton',
                                                                    props: {
                                                                        label: 'computeMostImportantFeatures',
                                                                        fullWidth: true,
                                                                        onClick: () => {
                                                                            this.loading.featureImportance = true;
                                                                            this.computeFeatureImportance()
                                                                            .catch((err) => console.error(err))
                                                                            .finally(() => {                                                                                
                                                                                this.loading.featureImportance = false;
                                                                            });                                                                            
                                                                        },
                                                                    },
                                                                },                                                                                                                            
                                                            ]       
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                }                             
                            ];

                            this.jobConfig.importantFeaturesSets.forEach((set, index) => {
                                let hidden = '';
                                if (set.dependentVar.uuid !== this.importantFeatureDependentVarUuid) {
                                    hidden = 'hidden';
                                }
                                vNodes.push(
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12 ' + hidden                                                        
                                    },
                                    children: {
                                        default:() => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'explanatoryDatapoints',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            const independentVarNodes = [];                                                 
                                                            set.independentVars.forEach((item, i) => {
                                                                const nodeName = item.uuid === '' ?  item.name : `${item.name} - ${item.pmModel.path}`;
                                                                // Translate time variables
                                                                const name = this.timeVars.includes(item.name) ? this.$t(item.name) : nodeName;
                                                                const percentage = (item.importance*100).toFixed(2);
                                                                independentVarNodes.push(
                                                                    {
                                                                        type: 'NgFormInput',
                                                                        props: {
                                                                            label: `[${percentage}%] ${name}`,
                                                                            type: 'checkbox',
                                                                            modelValue: this.jobConfig.importantFeaturesSets[index].independentVars[i].active,                        
                                                                            'onUpdate:modelValue': (value) => {
                                                                                this.jobConfig.importantFeaturesSets[index].independentVars[i].active = value;
                                                                            },                                 
                                                                        },    
                                                                    }                                                                     
                                                                );
                                                            });
                                                            return independentVarNodes;
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                });
                            });

                            vNodes.push(
                                { 
                                    type: 'NgLoading',
                                    props: {
                                        modelValue: this.loading.featureImportance,
                                    },
                                },                                
                            )
                            return vNodes;                                
                        }
                    }
                },                
                {
                    type: 'NgForm',
                    props: {
                        ref: 'jobForm',
                        greedy: true,
                        hideButtons: true,
                    },
                    children: {
                        default: (slot) => {                            
                            const vNodes = [
                                {
                                    type: 'NgFormInput',
                                    props: {
                                        col: 12,
                                        type: 'text',
                                        label: 'name',
                                        modelValue: this.jobLabel,
                                        'onUpdate:modelValue': (value) => {
                                            this.jobLabel = value;
                                        },
                                        rules: [
                                            (val) => val !== '' || this.$t('ruleNotEmpty')
                                        ]                                                                                             
                                    },    
                                },
                                {
                                    type: 'NgFormInput',
                                    props: {
                                        type: 'select',
                                        class: 'col-12',
                                        label: "executionTarget",
                                        modelValue: this.jobExecutionTarget,
                                        'onUpdate:modelValue': (value) => {
                                            this.jobExecutionTarget = value;
                                        },
                                        options: this.jobExecutionTargetOptions,
                                        rules: [
                                            (val) => (val !== '' &&  val !== undefined) || this.$t('ruleNotEmpty')
                                        ]                                                                                                                                                                   
                                    },    
                                },                                               
                                {
                                    type: 'NgFormInput',
                                    props: {
                                        type: 'select',
                                        class: 'col-12',
                                        label: "interval",
                                        modelValue: this.jobConfig.interval,
                                        'onUpdate:modelValue': (value) => {
                                            this.jobConfig.interval = value;
                                        },
                                        options: this.intervalOptions,
                                        rules: [
                                            (val) => (val !== '' &&  val !== undefined) || this.$t('ruleNotEmpty')
                                        ]                                                                                                                                                                   
                                    },    
                                },
                                {
                                    type: 'NgFormInput',
                                    props: {
                                        key: this.explanatoryDatapointsUpdateKey,
                                        label: 'dataPoints',
                                        col: '12',
                                        type: 'collection',
                                        defaultValue: this.nodeCollectionDefault,
                                        modelValue: this.jobConfig.nodes,
                                        enableSelectAll: false,
                                        'onAdd:item': () => {
                                            this.nodeCollectionDefault.dependentVariable = false;
                                            if (this.jobConfig.nodes.length === 0) {
                                                this.nodeCollectionDefault.dependentVariable = true;
                                            }
                                        },
                                        'onUpdate:modelValue': async (nodes) => {
                                            this.jobConfig.nodes = await Promise.all(nodes.map(async (node) => {
                                                node.extInfos = await this.ctx.intelli.fetchExtInfos(node.pmModel);
                                                if (node.uuid === ''){
                                                    node.uuid = this.ctx.uuidv4();
                                                }
                                                return node;
                                            }));
                                        },
                                        validators: [
                                            `
                                            (function() {                                                
                                                const dependentVariables = val.filter((node) => {
                                                    return (node['enabled'] === true && node['dependentVariable'] === true);                                                
                                                });     
                                                if (dependentVariables.length == 0) {
                                                    return $t('ruleAtLeastOneDependentVariable');
                                                }
                                                return true;
                                            })();
                                            `.replace(/(\r\n|\n|\r)/gm, ""),      
                                        ],
                                    },
                                    children: {
                                        form: (slot) => {  
                                            const node = slot.items[slot.selectedIndex];
                                            return [
                                                { 
                                                    type: 'NgFormInput',
                                                    props: {
                                                        type: 'checkbox',
                                                        col: 12,
                                                        label: 'enabled',
                                                        modelValue: node.enabled,
                                                        'onUpdate:modelValue': (value) => {
                                                            node.enabled = value;
                                                        },
                                                    },
                                                },                                                
                                                { 
                                                    type: 'NgFormInput',
                                                    props: {
                                                        type: 'checkbox',
                                                        col: 12,
                                                        label: 'defineAsdependentAndIndependentVariable',
                                                        modelValue: node.dependentVariable,
                                                        'onUpdate:modelValue': (value) => {
                                                            node.dependentVariable = value;
                                                        },
                                                    },
                                                },                                                                                            
                                                { 
                                                    type: 'NgFormField',
                                                    props: {
                                                        label: 'path',
                                                        type: 'dms-path',
                                                        class: 'col-12',
                                                        modelValue: node.pmModel,
                                                        filter: [{ hasHistData: true }],         
                                                        rules: [(val) => val !== '' ||this.$t('ruleNotEmpty')],
                                                        'onUpdate:modelValue': (value) => {
                                                            node.pmModel = value;
                                                        },                                                        
                                                    },    
                                                },
                                                { 
                                                    type: 'NgBIInterpolation',
                                                    props: {
                                                        modelValue: node.interpolation,
                                                        class: 'row col-12 q-col-gutter-sm',
                                                        'onUpdate:modelValue': (value) => {
                                                            node.interpolation = value;
                                                        },
                                                    },
                                                },    
                                                { 
                                                    type: 'NgBIBins',
                                                    props: {
                                                        modelValue: node.bins,
                                                        extInfos: node.extInfos,
                                                        pmModel: node.pmModel,
                                                        class: 'row col-12 q-col-gutter-sm',
                                                        'onUpdate:modelValue': (value) => {
                                                            node.bins = value;
                                                        },
                                                    },
                                                },   
                                                { 
                                                    type: 'NgFormInput',
                                                    props: {
                                                        type: 'checkbox',
                                                        label: 'mandatory',
                                                        modelValue: node.mandatory,
                                                        'onUpdate:modelValue': (value) => {
                                                            node.mandatory = value;
                                                        },
                                                    },
                                                },                                                                                                                                          
                                            ]
                                        },
                                        item: (slot) => {
                                            const node = slot.item;
                                            const interpolationName = this.$t(this.ctx.intelli.getInterpolationName(node.interpolation));
                                            const name = node.extInfos.name || '';
                                            const unit = node.extInfos.unit || node.unit || 'N/A';
                                            return [
                                                { 
                                                    type: 'NgText',
                                                    props: {
                                                        text:  [
                                                            name, `[${unit}]`, 
                                                            `${node.pmModel.path} /`, 
                                                            `${interpolationName} /`, 
                                                            ` ${node.dependentVariable ? this.$t('dependentVariable') : '' } /`,
                                                            node.enabled ? this.$t('enabled') : this.$t('disabled')
                                                        ],
                                                    },                                
                                                }, 
                                            ]
                                        },                                                                                                 
                                    },
                                },
                                {
                                    type: 'div',
                                    props: {
                                        class: 'row col-12',
                                    },
                                    children: {
                                        default: (slot) => {                            
                                            return [
                                                { 
                                                    type: 'NgActionButton',
                                                    props: {
                                                        label: 'visualizeDatapoint',
                                                        fullWidth: true,
                                                        disable: this.jobConfig.nodes.length !== 0 ? false : true,
                                                        color: this.jobConfig.nodes.length !== 0 ? 'primary' : 'grey',
                                                        onClick: () => this.loadChart(),
                                                    },
                                                },  
                                            ];     
                                        }
                                    }
                                },                                                                                                  
                                { 
                                    type: 'NgFormInput',
                                    props: {
                                        label: 'trainMethod',
                                        col: '12',
                                        type: 'select',
                                        modelValue: this.jobConfig.trainMethod,
                                        'onUpdate:modelValue': (value) => {
                                            this.jobConfig.trainMethod = value;
                                            this.selectionMethod = value;
                                            this.jobConfig.trainingDates = [];
                                            this.jobConfig.measureDates = [];                                            
                                            this.dateSelectorConfig[0].items[0].dates = [];
                                            this.dateSelectorConfig[1].items[0].dates = [];
                                            this.dateSelectorConfig[1].items[1].dates = [];                                            
                                        },
                                        rules: [(val) => val !== '' ||this.$t('ruleNotEmpty')],
                                        options: this.getTrainMethodOptions(),
                                    },
                                },  
                                { 
                                    type: 'NgFormInput',
                                    props: {
                                        'data-cy': 'trainingDates',
                                        label: this.jobConfig.trainMethod === 'movingWindows' ? 'movingWindowsEvaluationTimeSpan': 'isolatedWindowsTrainTimeSpan',
                                        col: '12',
                                        type: 'collection',
                                        required: true,
                                        modelValue: this.jobConfig.trainingDates,
                                        'onUpdate:modelValue': (values) => {
                                            this.jobConfig.trainingDates = values;
                                            this.dateSelectorConfig[0].items[0].dates = values;
                                            this.dateSelectorConfig[1].items[0].dates = values;
                                        },
                                        defaultValue: { start: '', end: ''},
                                        enableSelectAll: false,
                                        maxItems: this.jobConfig.trainMethod === 'movingWindows' ? 1 : undefined,
                                    },
                                    children: {
                                        form: (slot) => {
                                            const trainingDate = slot.items[slot.selectedIndex];
                                            return [
                                                { 
                                                    type: 'NgFormInput',
                                                    props: {
                                                        label: 'start',
                                                        col: '12',
                                                        type: 'date-time',
                                                        modelValue: trainingDate.start,
                                                        'onUpdate:modelValue': (value) => {
                                                            trainingDate.start = value;
                                                        },
                                                        time: false,
                                                        mask:'YYYY-MM-DD',
                                                        format:'YYYY-MM-DD',
                                                        rules:[
                                                        (val) => val !== ''  || this.$t('ruleNotEmpty'),
                                                        (val) => val < trainingDate.end || this.$t('ruleStartDateSmallThanEndDate'),
                                                        (val) => this.ctx.moment.moment(val, 'YYYY-MM-DD', true).isValid() || this.$t('ruleInvalidDateFormat', {'format': 'YYYY-MM-DD'}),
                                                        ]                                                    
                                                    },
                                                },
                                                { 
                                                    type: 'NgFormInput',
                                                    props: {
                                                        label: 'end',
                                                        col: '12',
                                                        type: 'date-time',
                                                        modelValue: trainingDate.end,
                                                        'onUpdate:modelValue': (value) => {
                                                            trainingDate.end = value;
                                                        },
                                                        time: false,
                                                        mask:'YYYY-MM-DD',
                                                        format:'YYYY-MM-DD',
                                                        rules:[
                                                            (val) => val !== ''  || this.$t('ruleNotEmpty'),
                                                            (val) => this.ctx.moment.moment(val, 'YYYY-MM-DD', true).isValid() || this.$t('ruleInvalidDateFormat', {'format': 'YYYY-MM-DD'}),
                                                        ]                                                    
                                                    },
                                                }                                                
                                            ]                                          
                                        },
                                        item: (slot) => {
                                            const trainingDates = slot.item;
                                            return [
                                                this.ctx.moment.moment(trainingDates.start).format('DD.MM.YYYY') + ' - ' +  this.ctx.moment.moment(trainingDates.end).format('DD.MM.YYYY'),
                                            ]
                                        },                                         
                                    }
                                },
                                {
                                    type: 'div',
                                    props: {
                                        class: 'row col-12',
                                    },
                                    children: {
                                        default: (slot) => {                            
                                            return [                                      
                                                // { 
                                                //     type: 'q-field',
                                                //     props: {
                                                //         // class
                                                //         label:'that label',
                                                //         error: true,
                                                //         'error-message': 'fefe',
                                                //         'reactiveRules':true,
                                                //         rules:[]
                                                //     },
                                                //     children: {
                                                //         default: () => {
                                                //             return [
                                                                { 
                                                                    type: 'NgActionButton',
                                                                    props: {
                                                                        label: 'mostImportantFeatures',
                                                                        fullWidth: true,
                                                                        disable: this.enableFeatureImporanceTest() ? false : true,
                                                                        color: this.enableFeatureImporanceTest() ? 'primary' : 'grey',                                                        
                                                                        onClick: () => {
                                                                            if (this.jobConfig.importantFeaturesSets.length == 0) {
                                                                                this.loading.main = true;
                                                                                this.computeFeatureImportance()
                                                                                .catch((err) => console.error(err))
                                                                                .finally(() => {
                                                                                    this.dialog.featureImportance = true;
                                                                                    this.loading.main = false;
                                                                                });  
                                                                            } else {
                                                                                this.dialog.featureImportance = true;
                                                                            }                                                                      
                                                                        },
                                                                    },
                                                                },
                                                //             ];                                                                         
                                                //         }
                                                //     }
                                                // },                                                  
                                            ];     
                                        }
                                    }
                                },                                                                                                                                                                             
                                { 
                                    type: 'div',
                                    props: {
                                        class: 'q-gutter-xs',
                                    },
                                    children: {
                                        default: () => {  
                                            return [
                                                { 
                                                    type: 'NgActionButton',
                                                    props: {
                                                        label: 'save',
                                                        class: 'col-4 q-gutter-xs',
                                                        onClick: () => this.validateAndSave(false),
                                                    },
                                                }, 
                                                { 
                                                    type: 'NgActionButton',
                                                    props: {
                                                        label: 'saveAndCompute',
                                                        class: 'col-4 q-gutter-xs',
                                                        onClick: () => this.validateAndSave(true),
                                                    },
                                                },                                                                                                
                                            ]                          
                                        }                                        
                                    }
                                },                                 
                            ]
                            
                            const movingWindowsNodes = [
                                {
                                    type: PeriodsDay,
                                    props: {
                                        class: 'row col-12',
                                        periodsDayClass: '',
                                        renderIgnoreEvents: false,
                                        modelValue: this.jobConfig,
                                        'onUpdate:modelValue': (value) => {
                                            this.jobConfig = value;
                                        }     
                                    },
                                },
                            ];

                            const isolatedWindowsNodes = [
                                { 
                                    type: 'NgFormInput',
                                    props: {
                                        label: 'measureOnFollowingPeriods',
                                        col: '12',
                                        type: 'collection',
                                        required: true,
                                        modelValue: this.jobConfig.measureDates,
                                        'onUpdate:modelValue': (values) => {
                                            this.jobConfig.measureDates = values;
                                            this.dateSelectorConfig[1].items[1].dates = values;                                            
                                        },
                                        defaultValue: { start: '', end: ''},
                                        enableSelectAll: false,
                                    },
                                    children: {
                                        form: (slot) => {
                                            const measureDate = slot.items[slot.selectedIndex];
                                            return [
                                                { 
                                                    type: 'NgFormInput',
                                                    props: {
                                                        label: 'start',
                                                        col: '12',
                                                        type: 'date-time',
                                                        modelValue: measureDate.start,
                                                        'onUpdate:modelValue': (value) => {
                                                            measureDate.start = value;
                                                        },
                                                        time: false,
                                                        mask:'YYYY-MM-DD',
                                                        format:'YYYY-MM-DD',
                                                        rules:[
                                                            (val) => val !== ''  || this.$t('ruleNotEmpty'),
                                                            (val) => val < measureDate.end || this.$t('ruleStartDateSmallThanEndDate'),
                                                            (val) => this.ctx.moment.moment(val, 'YYYY-MM-DD', true).isValid() || this.$t('ruleInvalidDateFormat', {'format': 'YYYY-MM-DD'}),
                                                        ]                                                    
                                                    },
                                                },
                                                { 
                                                    type: 'NgFormInput',
                                                    props: {
                                                        label: 'end',
                                                        col: '12',
                                                        type: 'date-time',
                                                        modelValue: measureDate.end,
                                                        'onUpdate:modelValue': (value) => {
                                                            measureDate.end = value;
                                                        },
                                                        time: false,
                                                        mask:'YYYY-MM-DD',
                                                        format:'YYYY-MM-DD',
                                                        rules:[
                                                            (val) => val !== ''  || this.$t('ruleNotEmpty'),
                                                            (val) => this.ctx.moment.moment(val, 'YYYY-MM-DD', true).isValid() || this.$t('ruleInvalidDateFormat', {'format': 'YYYY-MM-DD'}),
                                                        ]                                                    
                                                    },
                                                }                                                
                                            ]                                          
                                        },
                                        item: (slot) => {
                                            const measureDate = slot.item;
                                            return [
                                                this.ctx.moment.moment(measureDate.start).format('DD.MM.YYYY') + ' - ' +  this.ctx.moment.moment(measureDate.end).format('DD.MM.YYYY'),
                                            ]
                                        },                                         
                                    }
                                },                                
                            ];
                            
                            if (this.jobConfig.trainMethod === 'movingWindows') {
                                vNodes.splice(6, 0, ...movingWindowsNodes);

                            } else {
                                vNodes.splice(7, 0, ...isolatedWindowsNodes);
                            }

                            return vNodes;
                        }
                    },
                },
                { 
                    type: 'NgLoading',
                    props: {
                        modelValue: this.loading.main,
                    },
                },                  
            ]
        },
    }
  });