const NodeCollection = window.nguiAsyncUrlImport('/service/intelli/v1/filedb/lib/NodeCollection.js');
const TrainingDatesCollection = window.nguiAsyncUrlImport('/service/intelli/v1/filedb/scripts/maxpermissiblevalues/components/trainingDateCollection.js');
const TrainDateSelector = window.nguiAsyncUrlImport('/service/intelli/v1/filedb/scripts/maxpermissiblevalues/components/trainDateSelector.js');
const PeriodsDay = window.nguiAsyncUrlImport('/service/intelli/v1/filedb/scripts/maxpermissiblevalues/components/periodsDay.js');
const NodeChartOptions = window.nguiAsyncUrlImport('/service/intelli/v1/filedb/lib/NodeChartOptions.js');

window.ngui(() => {
    return {        
        emits: [
            'form:submit',
        ],
        props: [
            'payload', 
        ],
        methods: {
            getMonthDelta(interval) {
                return interval < 0 ? 36 : 3;
            },
            togglePeriodsDayVisibility(trainMethod) {
                this.periodsDayClass = trainMethod === 'movingWindows' ? '' : 'hidden';
            },
            resetTrainingDates() {
                this.configModel.trainingDates = [];
                this.localTrainDates = [];
            },
            hasExplanatoryVariables() {
                let hasExplanatoryVariables = false
                if (this.configModel.explanatoryDateTimeVariables) {
                    if (this.configModel.explanatoryDateTimeVariables.length > 0) {
                        hasExplanatoryVariables = true
                    }
                }

                const explVars = this.independentNodes.filter((node) => {
                    if ('purposes' in node) {
                        return node.purposes.includes('computeThresholds') || false;
                    }
                    return false;
                });
                if (explVars.length > 0) {
                    hasExplanatoryVariables = true;
                }

                if (this.configModel.holidaysCountry !== '') {
                    hasExplanatoryVariables = true
                }                
                return hasExplanatoryVariables;
            }           
        },
        // The returned object can be accessed with this within any method...
        // Do not use this.ctx within this function because it is not yet created.
        data() {
            return {
                configModelLoaded: false,
                intervalOptions:  [
                    {label: 'hourly', value: 3600},
                    {label: 'daily', value: -1},
                    // {label: 'weekly', value: -2},
                ],
                dependentNode: {},
                independentNodes: [],
                localTrainMethod: 'movingWindows',
                localTrainDates: [],
                periodsDayClass: '',
                explanatoryDateTimeOptions: [
                    {label: 'hod', value: 'hod'},
                    {label: 'dow', value: 'dow'},
                    {label: 'month', value: 'month'},                    
                    {label: 'quarter', value: 'quarter'},                    
                    {label: 'Mo2Sa-So', value: 'Mo2Sa-So'},
                    {label: 'Mo2Fr-Sa-So', value: 'Mo2Fr-Sa-So'},        
                    {label: 'Mo2Fr-Sa2So', value: 'Mo2Fr-Sa2So'},
                ],
                holidaysOptions: [
                    {label: '', value: ''},
                    {label: 'holidaySwitzerland', value: 'CH'},
                    {label: 'holidayLuxembourg', value: 'LU'},
                ],                
                dialog: {
                    chart: false,
                    dependentNodeOptions: false,
                },
                trainingDateFormat: 'YYYY-MM-DD',
                trainMethodOptions: [
                    {label: 'movingWindows', value: 'movingWindows'},
                    {label: 'isolatedWindows', value: 'isolatedWindows'}                
                ],
                configModel: {
                    uuid: '',
                    task: {
                        name: 'maxpermissiblevalues',
                        algorithm: {
                            method: '',
                            model: '',
                        },
                        metric: undefined,
                    }, 
                    interval: -1,
                    nodes: [],
                    dependentVariables: [],
                    independentVariables: [],
                    explanatoryDateTimeVariables: [],
                    holidaysCountry: '',
                    minElementsPerBin: 3,
                    trainMethod: 'movingWindows',
                    trainTime: '00:00', 
                    trainingDates: [],
                    measureDates: [],
                    periodsDay: {
                        train: 60,
                        retrain: 20,
                        ignoreEvents: false,
                    },
                    sensitivityThreshold: {
                        upper: 1,
                        lower: 1,
                    },
                    eventCount: {
                        upper: 3,
                        lower: 3,
                    },
                    minErrorDistance: {
                        upper: 0,
                        lower: 0,
                    },
                    thresholdColor: '#99ff99',
                    expectedValueColor: '#47b547',
                },                        
                modelOptions: [                                      
                    '',
                    'AdaBoostRegressor',
                    'BaggingRegressor',
                    'DecisionTreeRegressor',
                    'ExtraTreesRegressor',
                    'GammaRegressor',
                    'GaussianProcessRegressor',
                    'GradientBoostingRegressor',
                    'HistGradientBoostingRegressor',
                    'HuberRegressor',
                    'KNeighborsRegressor',
                    'KernelRidge',
                    'Lars',
                    'LarsCV',
                    'LassoCV',
                    'LassoLarsCV',
                    'LassoLarsIC',
                    'LinearRegression',
                    'LinearSVR',
                    'MLPRegressor',
                    'NuSVR',
                    'OrthogonalMatchingPursuit',
                    'OrthogonalMatchingPursuitCV',
                    'PiecewiseRegressor',                    
                    'RANSACRegressor',
                    'RadiusNeighborsRegressor',
                    'RandomForestRegressor',
                    'RidgeCV',
                    'SVR',
                    'TheilSenRegressor',
                    'XGBoost',
                ] 
            }       
        },
        beforeMount() {
            this.widget = this.payload.widget;
            const now = this.ctx.moment.moment();
            const defaultTrainingDates = {
                start: '',
                end: now.format(this.trainingDateFormat)
            }
            const months = this.getMonthDelta(this.configModel.interval);
            defaultTrainingDates.start = now.subtract(months, 'months').format(this.trainingDateFormat);

            this.configModel.trainingDates.push(defaultTrainingDates);
            this.dependentNode = this.ctx.intelli.initNode();
            this.dependentNode.uuid = this.ctx.uuidv4();
                        
            // Get config model
            this.ctx.filedb.read('app/BAI/configModel/'+ this.widget.configModelUuid + '.json', true)
            .then((response) => {
                const configModel = response.data.data;
                this.configModel = {...this.configModel, ...configModel};

                const dependentNodes = this.configModel.nodes.filter((node) => {
                    return (this.configModel.dependentVariables.includes(`${node.pmModel.processUuid}_${node.pmModel.path}`));
                });
                if (dependentNodes.length > 0) {
                    this.dependentNode = dependentNodes[0];
                }                

                this.independentNodes = this.configModel.nodes.filter((node) => {
                    return (this.configModel.independentVariables.includes(`${node.pmModel.processUuid}_${node.pmModel.path}`));
                });

                this.localTrainMethod = this.configModel.trainMethod;
                this.togglePeriodsDayVisibility(this.configModel.trainMethod);                

                this.configModelLoaded = true;
            })
            .catch(() => {
                this.togglePeriodsDayVisibility(this.configModel.trainMethod);                    
            });                
        },
        // Define elements within the ui configuration...
        nodes() {
            return [
                {
                    type: 'NgDialog',
                    props: {
                        title: '',
                        maximizing: true,
                        modelValue: this.dialog.chart,
                        'onUpdate:modelValue': (value) => {
                            this.dialog.chart = value;
                        },                        
                        width: '1000px',
                        medium: false,
                        noOk: true,
                        noCancel: true,
                        draggable: true,
                        onHide: () => {
                            this.configModel.trainMethod = this.localTrainMethod;
                            this.configModel.trainingDates = this.localTrainDates;
                        },
                    },
                    children: {
                        default: (slot) => {                            
                            return [   
                                {
                                    type: TrainDateSelector,
                                    props: {
                                        trainMethodOptions: this.trainMethodOptions,
                                        trainMethod: this.configModel.trainMethod,
                                        trainDates: this.configModel.trainingDates,
                                        nodeCollection: this.configModel.nodes,
                                        interval: this.configModel.interval,
                                        intervalOptions: this.intervalOptions,                        
                                        'onUpdate:interval': (value) => {
                                            this.configModel.interval = value;
                                        },                                        
                                        'onUpdate:nodeCollection': (values) => {
                                            this.configModel.nodes = values;
                                        },
                                        'onUpdate:trainDates': (trainDates) => {
                                            this.localTrainDates = trainDates;                                                   
                                        },                                   
                                        'onUpdate:trainMethod': (value) => {
                                            this.localTrainMethod = value;
                                            this.togglePeriodsDayVisibility(value);
                                        },                                                                             
                                    }
                                }
                            ];
                        }
                    }
                },
                {
                    type: 'NgDialog',
                    props: {
                        title: this.dependentNode.pmModel.path,
                        maximizing: true,
                        modelValue: this.dialog.dependentNodeOptions,
                        'onUpdate:modelValue': (value) => {
                            this.dialog.dependentNodeOptions = value;
                        },                        
                        width: '1000px',
                        medium: false,
                        noOk: true,
                        noCancel: true,
                        draggable: true,
                    },
                    children: {
                        default: (slot) => {                            
                            return [
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:() => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'datapoint',
                                                    },
                                                    children: {
                                                        default: () => {
                                                            return [
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        label: 'unit',
                                                                        type: 'text',
                                                                        disable: !(this.dependentNode.extInfos && ('unit' in this.dependentNode.extInfos == false || this.dependentNode.extInfos.unit === '')),
                                                                        hint: this.$t('dmsUnit', {dmsUnit: this.dependentNode.extInfos?.unit || 'N/A'}),
                                                                        class: 'col-12',
                                                                        modelValue: this.dependentNode.unit,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.dependentNode.unit = value;
                                                                        },                                         
                                                                    },
                                                                },                                                                
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },                                
                                { 
                                    type: 'NgFormGroups',
                                    props: {
                                        class: 'col-12',
                                    },
                                    children: {
                                        default: (slot) => {
                                            return [
                                                { 
                                                    type: 'NgFormGroupItem',
                                                    props: {
                                                        title: "Interpolation",
                                                    },
                                                    children : {
                                                        default: () => {
                                                            return [
                                                                { 
                                                                    type: 'NgBIInterpolation',
                                                                    props: {
                                                                        modelValue: this.dependentNode.interpolation,
                                                                        class: 'row col-12 q-col-gutter-sm',
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.dependentNode.interpolation = value;
                                                                        },
                                                                    },
                                                                },
                                                            ];
                                                        }
                                                    }
                                                },  
                                            ];
                                        }
                                    }
                                },                                  
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'scale',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                { 
                                                                    type: NodeChartOptions,
                                                                    props: {
                                                                        class: 'q-col-gutter-sm row',
                                                                        modelValue: this.dependentNode,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.dependentNode = value;
                                                                        },
                                                                        // 'onUpdate:skipComputation': (value) => {
                                                                        //     this.$emit('update:skipComputation', value);
                                                                        // },                                                                        
                                                                    },        
                                                                },                                                               
                                                            ]
                                                        }
                                                    }
                                                },
                                                {
                                                    type: 'NgActionButton',
                                                    props: {
                                                        label: 'save',
                                                        onClick: () => {
                                                            this.dialog.dependentNodeOptions = false;
                                                        }    
                                                    },
                                                },                                                   
                                            ]
                                        }
                                    }
                                },  
                            ];
                        }
                    }
                },                          
                {
                    type: 'NgForm',                
                    props: {
                        greedy: true,
                        submitLabel: 'computeThresholds',
                        'data-cy': 'computeThresholdsForm',
                        key: this.configModelLoaded,
                        onSubmit: async () =>  {
                            try {
                                this.configModel.nodes = [...this.independentNodes, this.dependentNode];
                                // Set transform dates from latest end date to today
                                if (this.configModel.trainMethod === 'isolatedWindows') {
                                    const trainingEndDates = this.configModel.trainingDates.map((date) => {
                                        return date.end;
                                    })
                                    const latestEndDate = trainingEndDates.reverse()[0];
                                    const today = this.ctx.moment.moment().format('YYYY-MM-DD');
                                    this.configModel.measureDates = [{
                                        'start': latestEndDate,
                                        'end': today
                                    }];
                                }

                                this.configModel.dependentVariables = [`${this.dependentNode['pmModel']['processUuid']}_${this.dependentNode['pmModel']['path']}`];

                                // Update independentVariables
                                this.configModel.independentVariables = [];                    
                                this.independentNodes.forEach((node) => {
                                    if ('purposes' in node && node['purposes'].includes('computeThresholds')) {
                                        const nodeIdentifier = `${node['pmModel']['processUuid']}_${node['pmModel']['path']}`
                                        this.configModel.independentVariables.push(nodeIdentifier);
                                    }
                                })
                                this.configModel.independentVariables = this.configModel.independentVariables.concat(this.configModel.explanatoryDateTimeVariables);

                                // 6.5.24 todo rethink holiday handling
                                if (this.configModel.holidaysCountry !== '') {
                                    this.configModel.independentVariables.push('holiday');
                                }

                                // set uuids
                                this.configModel.nodes = this.configModel.nodes.map((node) => {
                                    if (!node.uuid) {
                                        node.uuid = this.ctx.uuidv4();
                                    }
                                    return node;
                                });

                                await this.ctx.filedb.insert('app/BAI/configModel/'+ this.configModel.uuid + '.json', this.configModel);                                

                                const nodes = this.configModel.nodes?.map((node) => {
                                    const dependentVariableNames = this.configModel.dependentVariables;
                                    return {
                                        uuid: node.uuid,                               
                                        dependentVariable: (dependentVariableNames.includes(`${node.pmModel.processUuid}_${node.pmModel.path}`)),
                                        interpolation: node.interpolation,
                                        bins: node.bins,
                                        pmModel: node.pmModel,
                                        extInfos: node.extInfos,
                                        chartOptions: node.chartOptions,
                                    };
                                  });

                                  const nodeOptions = this.configModel.nodes?.map((node) => {
                                    return {
                                      name: this.ctx.intelli.getNodeChartName(node),
                                      type: 'line',
                                      data: {
                                        yUuidRef: node.uuid,
                                      },
                                      yAxis: this.ctx.intelli.getNodeChartName(node),
                                    }
                                  });

                                
                                const trainingDates = this.configModel.trainingDates.map((date) => this.ctx.intelli.getLocalizedDateTimeByInterval(date, this.configModel.interval));

                                let measureDates = [];
                                if (this.configModel.measureDates && this.configModel.measureDates.length >  0) {                              
                                    measureDates = this.configModel.measureDates.map((date) => this.ctx.intelli.getLocalizedDateTimeByInterval(date, this.configModel.interval));
                                }

                                const years = this.ctx.intelli.getYearListFromDates(trainingDates.concat(measureDates));
                                const holidays = this.ctx.intelli.getHolidays(this.configModel.holidaysCountry, years, ['public']);                                

                                const dependentNode = this.configModel.nodes?.filter((node) => this.configModel.dependentVariables?.includes(`${node['pmModel']['processUuid']}_${node['pmModel']['path']}`))[0];

                                const payload = {
                                    scriptType: 'maxpermissiblevalues',
                                    scriptFile: 'compute_thresholds.py',
                                    scriptArguments: {
                                        configModelUuid: this.configModel.uuid,
                                        trainingDates,
                                        measureDates,
                                        dependentNodeName: this.ctx.intelli.getNodeChartName(dependentNode)
                                    },
                                    dates: trainingDates.concat(measureDates),
                                    nodes,
                                    nodeOptions,
                                    holidays: holidays,
                                    histData: {
                                        start: '',
                                        end: '',
                                        interval: this.configModel.interval,
                                    }
                                }

                                if (this.widget.name === '') {
                                    this.widget.name = this.dependentNode['pmModel']['path'];
                                    if (this.dependentNode['extInfos']['name'] !== '') {
                                        this.widget.name = `${this.dependentNode['extInfos']['name']} - ${this.dependentNode['pmModel']['path']}`;
                                    }
                                }
                                await this.ctx.filedb.insert('app/BAI/widgets/'+ this.widget.uuid + '.yaml', this.widget);                                

                                this.$emit('form:submit', payload);
                            } catch (err) {
                                console.error(err);
                            }
                        },
                    },
                    children: {
                        default: () => {
                            return [
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:() => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'observedDatapoint',
                                                    },
                                                    children: {
                                                        default: () => {
                                                            return [
                                                                { 
                                                                    type: 'NgDMSPathChooser',
                                                                    props: {
                                                                        'data-cy': 'PathChooserInput',
                                                                        label: 'path',
                                                                        required: true,
                                                                        readonly: true,
                                                                        filter: [{ hasHistData: true }],
                                                                        modelValue: this.dependentNode.pmModel,
                                                                        class: 'col-12',
                                                                        'onUpdate:modelValue': async (value) => {
                                                                            this.dependentNode.pmModel = value;
                                                                            const extInfos = await this.ctx.intelli.fetchExtInfos(value);  
                                                                            this.dependentNode.extInfos = extInfos;
                                                                            // keep this.configModel.nodes update to date for trainDateSelector
                                                                            this.configModel.nodes = [this.dependentNode, ...this.independentNodes];                                                                            
                                                                        },
                                                                        rules: [
                                                                            (val) => val !== '' || this.$t('ruleNotEmpty'),
                                                                        ]
                                                                    },
                                                                },
                                                                {
                                                                    type: 'NgActionButton',
                                                                    props: {
                                                                        label: 'chartOptions',
                                                                        fullWidth: true,
                                                                        disable: this.dependentNode.pmModel.path === '' ? true : false,
                                                                        onClick: () => {
                                                                            this.dialog.dependentNodeOptions = true;
                                                                        }    
                                                                    },
                                                                },                                                                   
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12',
                                        separator: false,
                                        bordered: false,
                                        dense: true,

                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'timeResolution',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'interval',
                                                                        type: 'select',
                                                                        options: this.intervalOptions,
                                                                        modelValue: this.configModel.interval,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.interval = value;
                                                                            if (this.configModel.trainingDates.length > 0) {
                                                                                const months = this.getMonthDelta(this.configModel.interval);
                                                                                // Set default training dates on time resolution update.
                                                                                if (this.configModel.trainMethod === 'movingWindows') {
                                                                                    let endDate = this.ctx.moment.getLocalizedNow();
                                                                                    if (this.configModel.trainingDates[0].end !== '') {
                                                                                        endDate = this.ctx.moment.getDate(this.configModel.trainingDates[0].end)
                                                                                    }
                                                                                    this.configModel.trainingDates[0].start = endDate.subtract(months, 'months').format('YYYY-MM-DD');                                            
                                                                                }
                                                                            }
                                                                        },
                                                                        rules: [(val) => val !== '' && val !== undefined || this.$t('ruleNotEmpty')]                                                                             
                                                                    },
                                                                },                                                                   
                                                            ]
                                                        }
                                                    }
                                                }, 
                                            ]
                                        },
                                    }
                                },                                         
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'algorithms',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [                                                               
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 6,
                                                                        label: 'method',                                        
                                                                        type: 'select',
                                                                        options: [
                                                                            {
                                                                                label: '',
                                                                                value: ''
                                                                            },                                            
                                                                            {
                                                                                label: 'thresholdsRegression',
                                                                                value: 'thresholdsRegression'
                                                                            },
                                                                            {
                                                                                label: 'thresholdsGrouping',
                                                                                value: 'thresholdsGrouping'
                                                                            }                                               
                                                                        ],
                                                                        modelValue: this.configModel.task.algorithm.method,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            if (value !== 'thresholdsRegression') {
                                                                                this.configModel.task.algorithm.model = '';
                                                                            }
                                                                            this.configModel.task.algorithm.method = value;                                            
                                                                        },
                                                                        rules: [
                                                                            (val) => (val !== undefined && val !== '') || this.$t('ruleNotEmpty')
                                                                        ]                                                                             
                                                                    },
                                                                },
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 6,
                                                                        label: 'model',                                        
                                                                        type: 'select',
                                                                        disable: !(this.configModel.task.algorithm.method === 'thresholdsRegression'),                                        
                                                                        options: this.modelOptions.map((option) => { 
                                                                            return { label: option, value: option };
                                                                        }),
                                                                        modelValue: this.configModel.task.algorithm.model,                                                                          
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.task.algorithm.model = value;
                                                                        },
                                                                        rules: [
                                                                            (val) => (this.configModel.task.algorithm.method === 'thresholdsRegression' && (val !== undefined && val !== '')) || this.$t('ruleNotEmpty')
                                                                        ]                                                                                                      
                                                                    },
                                                                },                                                                 
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },                                                                                                                                                                                                                                                                                                                                                                                              
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12',
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [                                                    
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'explanatoryVariables',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                {
                                                                    type: NodeCollection,
                                                                    props: {
                                                                        modelValue: this.independentNodes,
                                                                        purposeOptions: ['computeThresholds'],
                                                                        selectedAlgo: this.configModel.task.algorithm,
                                                                        'onUpdate:modelValue': (values) => {
                                                                            this.independentNodes = values;
                                                                            // keep this.configModel.nodes update to date for trainDateSelector
                                                                            this.configModel.nodes = [this.dependentNode, ...values];
                                                                        },                                                                          
                                                                        datapointListRules: [
                                                                            // this.hasExplanatoryVariables() ? 'true' : '$t("ruleAtLeastOneExplanatoryVariableWithPurpose")', // todo Fix the line!
                                                                            `
                                                                            (function() {
                                                                                if (attrs.selectedAlgo.method === 'thresholdsGrouping') {
                                                                                    const errors = val.filter((node) => {
                                                                                        if (node['extInfos']['accType'] === 'double64') {
                                                                                            if ('bins' in node) {
                                                                                                if ('base' in node['bins'] === false || 'step' in node['bins'] === false) {
                                                                                                    return true;
                                                                                                }
                                                                                                if ([node['bins']['base'], node['bins']['step']].includes(undefined)) {
                                                                                                    return true;
                                                                                                }
                                                                                            }
                                                                                        }                                                                                     
                                                                                    });     
                                                                                    if (errors.length > 0) {
                                                                                        return $t('ruleAlgoNeedsBinning');
                                                                                    }
                                                                                }
                                                                                return true;
                                                                            })();
                                                                            `.replace(/(\r\n|\n|\r)/gm, ""),                                                                                                                                             
                                                                        ]
                                                                    },
                                                                },                                                                
                                                                {
                                                                    type: 'NgFormInput',                                
                                                                    props: {
                                                                        col: 6,
                                                                        label: 'dateTimeVariables',
                                                                        type: 'select',
                                                                        multiple: true,
                                                                        options: this.explanatoryDateTimeOptions,
                                                                        modelValue: this.configModel.explanatoryDateTimeVariables,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.explanatoryDateTimeVariables = value;
                                                                        },
                                                                        rules: [
                                                                            (val) => this.hasExplanatoryVariables() || this.$t("ruleAtLeastOneExplanatoryVariableWithPurpose")
                                                                        ]                                                                       
                                                                    },
                                                                },
                                                                {
                                                                    type: 'NgFormInput',                                
                                                                    props: {
                                                                        col: 6,
                                                                        label: 'holidays',
                                                                        type: 'select',
                                                                        options: this.holidaysOptions,
                                                                        modelValue: this.configModel.holidaysCountry,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.holidaysCountry = value;
                                                                        }                                                                    
                                                                    },
                                                                },                                                                  
                                                                // {
                                                                //     type: 'NgFormInput',
                                                                //     props: {
                                                                //         col: 6,
                                                                //         label: 'minElementsPerBin',
                                                                //         type: 'number',
                                                                //         modelValue: this.configModel.minElementsPerBin,
                                                                //         'onUpdate:modelValue': (value) => {
                                                                //             this.configModel.minElementsPerBin = value;
                                                                //         },
                                                                //         rules: [(val) => val !== "" || this.$t("ruleNotEmpty")]                                                                          
                                                                //     },
                                                                // },                                                                                                                               
                                                            ];
                                                        }
                                                    }
                                                },  
                                            ]
                                        }
                                    }                                    
                                },
                                {
                                    type: 'NgFormGroups',                                
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'timeSpans',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [                                                               
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col:12,
                                                                        label: 'trainMethod',
                                                                        type: 'select',
                                                                        modelValue: this.configModel.trainMethod,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.trainMethod = value;
                                                                            this.localTrainMethod = value;
                                                                            this.togglePeriodsDayVisibility(value);
                                                                            this.resetTrainingDates();
                                                                        },
                                                                        options: this.trainMethodOptions,
                                                                        rules: [
                                                                            (val) => val !== '' || this.$t('ruleNotEmpty'),
                                                                            () => this.configModel.trainingDates.length > 0 || this.$t('ruleMinElement', {total: 1}),
                                                                        ]
                                                                    },
                                                                },                                                             
                                                                {
                                                                    type: 'div',
                                                                    props: {
                                                                        class: 'row col-12',
                                                                    },
                                                                    children: {
                                                                        default: (slot) => {                            
                                                                            return [
                                                                                {
                                                                                    type: 'NgActionButton',
                                                                                    props: {
                                                                                        fullWidth: true,
                                                                                        color: this.dependentNode.pmModel?.path ? 'primary' : 'grey',
                                                                                        disable: this.dependentNode.pmModel?.path ? false : true,
                                                                                        label: 'visualizeDatapoint',
                                                                                        onClick: () => {
                                                                                            this.dialog.chart = true;
                                                                                        }    
                                                                                    },
                                                                                },                                                                                    
                                                                            ]
                                                                        }
                                                                    }
                                                                },                                                              
                                                                {
                                                                    type: TrainingDatesCollection,
                                                                    props: {
                                                                        class: 'col-12',
                                                                        trainMethod: this.configModel.trainMethod,
                                                                        modelValue: this.configModel.trainingDates,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.trainingDates = value;
                                                                        },                                        
                                                                    },
                                                                },                                                            
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },                                 
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12 ' + this.periodsDayClass                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'trainingPeriod',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            const trainingPeriodRules = [];
                                                            if (this.configModel.task.algorithm.method === 'thresholdsGrouping') {
                                                                trainingPeriodRules.push(
                                                                    () => {
                                                                        if (this.configModel.explanatoryDateTimeVariables.includes('quarter')
                                                                            && this.configModel.periodsDay.train <= 90) {
                                                                                return 'thresholdsGroupingAndQuarterAtLeast90Days';
                                                                        }
                                                                        if (this.configModel.explanatoryDateTimeVariables.includes('month')
                                                                            && this.configModel.periodsDay.train <= 30) {
                                                                                return 'thresholdsGroupingAndMonthAtLeast30Days';
                                                                        }                                    
                                                                        return true;
                                                                    });
                                                            }
                                                            return [
                                                                {
                                                                    type: PeriodsDay,
                                                                    props: {
                                                                        class: 'row',
                                                                        modelValue: this.configModel,
                                                                        renderIgnoreEvents: true,
                                                                        trainingPeriodRules,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel = value;
                                                                        }                                                                     
                                                                    },
                                                                },                                                                
                                                            ];
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'sensitivity',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col:12,
                                                                        label: 'upper',
                                                                        type: 'slider',
                                                                        modelValue: this.configModel.sensitivityThreshold.upper,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.sensitivityThreshold.upper = value;
                                                                        },
                                                                        min: 1,
                                                                        max: 100,
                                                                        steps: 1,
                                                                        reverse: true,
                                                                        required: true,
                                                                        default: 1,                          
                                                                    },
                                                                },   
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col:12,
                                                                        label: 'lower',
                                                                        type: 'slider',
                                                                        min: 1,
                                                                        max: 100,
                                                                        steps: 1,
                                                                        required: true,
                                                                        default: 1,                                   
                                                                        modelValue: this.configModel.sensitivityThreshold.lower,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.sensitivityThreshold.lower = value;
                                                                        },
                                                                    },
                                                                },                                                                
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'eventCount',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'upper',
                                                                        type: 'number',
                                                                        modelValue: this.configModel.eventCount.upper,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.eventCount.upper = value;
                                                                        },
                                                                        rules: [(val) => val !== '' && val !== undefined || this.$t('ruleNotEmpty'), (val) => val >= 0 || this.$t('ruleAbsolute')]
                                                                    },
                                                                },   
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'lower',
                                                                        type: 'number',
                                                                        modelValue: this.configModel.eventCount.lower,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.eventCount.lower = value;
                                                                        },
                                                                        rules: [(val) => val !== '' && val !== undefined  || this.$t('ruleNotEmpty'), (val) => val >= 0 || this.$t('ruleAbsolute')]
                                                                    },
                                                                },                                                                
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'minErrorDistance',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'upper',
                                                                        type: 'number',
                                                                        modelValue: this.configModel.minErrorDistance.upper,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.minErrorDistance.upper = value;
                                                                        },
                                                                        step: 'any',
                                                                        rules: [(val) => val !== '' && val !== undefined || this.$t('ruleNotEmpty'), (val) => val >= 0 || this.$t('ruleAbsolute')]
                                                                    },
                                                                },
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'lower',
                                                                        type: 'number',
                                                                        modelValue: this.configModel.minErrorDistance.lower,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.minErrorDistance.lower = value;
                                                                        },
                                                                        step: 'any',
                                                                        rules: [(val) => val !== '' && val !== undefined || this.$t('ruleNotEmpty'), (val) => val >= 0 ||this.$t('ruleAbsolute')]
                                                                    },
                                                                },                                                                
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },                
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'visualization',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'thresholdColor',
                                                                        type: 'color',
                                                                        modelValue: this.configModel.thresholdColor,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.thresholdColor = value;
                                                                        },
                                                                        rules: [(val) => val !== '' || this.$t('ruleNotEmpty')]
                                                                    },
                                                                },
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'expectedValueColor',
                                                                        type: 'color',
                                                                        modelValue: this.configModel.expectedValueColor,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.expectedValueColor = value;
                                                                        },
                                                                        rules: [(val) => val !== '' || this.$t('ruleNotEmpty')]
                                                                    },
                                                                },                                                                                                                                                                              
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                }                                                                                               
                            ]                                            
                        },
                    },                   
                }
            ]
        },
    }
});